//
// Created by daniel on 21.5.20.
//

#ifndef PROGRAM_LATCH_H
#define PROGRAM_LATCH_H

/*
 * --------------------------------------------
 * Blif file - Latch
 * --------------------------------------------
 *
 * A generic-latch is used to create a delay element in a model. It represents one bit of memory or state information.
 * The generic-latch construct can be used to create any type of latch or flip-flop (see also the library-gate section).
 * A generic-latch is declared as follows:
 *
 * .latch <input> <output> [<type> <control>] [<init-val>]
 *
 * input is the data input to the latch.
 *
 * output is the output of the latch.
 *
 * type is one of f fe, re, ah, al, as g , which correspond to “falling edge,” “rising edge,” “active high,” “active low,”
 * or “asynchronous.”
 *
 * control is the clocking signal for the latch. It can be a .clock of the model, the output of any function in the model,
 * or the word “NIL” for no clock.
 *
 * init-val is the initial state of the latch, which can be one of f 0, 1, 2, 3 g . “2” stands for “don’t care” and “3” is
 * “unknown.” Unspecified, it is assumed “3.”
 *
 * If a latch does not have a controlling clock specified, it is assumed that it is actually controlled by a single
 * global clock. The behavior of this global clock may be interpreted differently by the various algorithms that may
 * manipulate the model after the model has been read in. Therefore, the user should be aware of these varying
 * interpretations if latches are specified with no controlling clocks.
 *
 * Important: All feedback loops in a model must go through a generic-latch. Purely combinational-logic cycles
 * are not allowed.
 *
 * Example:
 * .inputs d
 * .output q
 * .clock c
 * .latch d q re c 0
 * .end # a clocked flip-flop
 * .inputs in
 * .outputs out
 * .latch out in 0
 * .names in out
 * 0 1
 * .end
 *
 */

#include "../command.h"

class Latch : public Command {
public:
    Latch()
    : Command(LATCH)
    {
    }

    Latch(const std::string & name)
    : Command(name, LATCH)
    {
    }

    std::shared_ptr<Command> Clone () const {
        return std::make_shared<Latch>(*this);
    }

    void AddParam(const std::string & param) {
        m_Params.push_back(param);
    }

private:
    std::vector<std::string> m_Params;
};
typedef std::shared_ptr<Latch> ALatch;

#endif //PROGRAM_LATCH_H
