//
// Created by daniel on 21.5.20.
//

#ifndef PROGRAM_LOGICGATE_H
#define PROGRAM_LOGICGATE_H

/*
 * --------------------------------------------
 * Blif file - Logic Gate
 * --------------------------------------------
 *
 * A logic-gate associates a logic function with a signal in the model, which can be used as an input to other logic
 * functions. A logic-gate is declared as follows:
 *
 * .names <in-1> <in-2> ... <in-n> <output>
 * <single-output-cover>
 *
 * output is a string giving the name of the gate being defined.
 *
 * in-1, in-2, ... in-n are strings giving the names of the inputs to the logic gate being defined.
 *
 * single-output-cover is, formally, an n-input, 1-output PLA description of the logic function corresponding to the
 * logic gate. f 0, 1, – g is used in the n-bit wide “input plane” and f 0, 1 g is used in the 1-bit wide “output
 * plane”. The ON -set is specified with 1’s in the “output plane,” and the OFF -set is specified with 0’s in the
 * “output plane.” The DC -set is specified for primary output nodes only, by using the construct .exdc.
 *
 * A sample logic-gate with its single-output-cover:
 *
 * .names v3 v6 j u78 v13.15
 * 1--0 1
 * -1-1 1
 * 0-11 1
 *
 * In a given row of the single-output-cover, “1” means the input is used in uncomplemented form, “0” means
 * the input is complemented, and “–” means not used. Elements of a row are AND ed together, and then all rows are
 * OR ed.
 *
 * As a result, if the last column (the “output plane”) of the single-output-cover is all 1’s, the first n columns (the
 * “input plane”) of the single-output-cover can be viewed as the truth table for the logic gate named by the string
 * output. The order of the inputs in the single-output-cover is the same as the order of the strings in-1, in-2, ..., in-n
 * in the .names line. A space between the columns of the “input plane” and the “output plane” is required.
 * The translation of the above sample logic-gate into a sum-of-products notation would be as follows:
 *
 * v13.15 = (v3 u78’) + (v6 u78) + (v3’ j u78)
 *
 * To assign the constant “0” to some logic gate j, use the following construct:
 * .names j
 *
 * To assign the constant “1”, use the following:
 * .names j
 * 1
 *
 * The string output can be used as the input to another logic-gate before the logic-gate for output is itself defined.
 * For a more complete description of the PLA input format, see espresso(5).
 *
 */

#include "../command.h"

class LogicGate : public Command {
public:
    LogicGate()
    : Command(LOGIC_GATE)
    {
    }

    LogicGate(const std::string & name)
    : Command(name, LOGIC_GATE)
    {
    }


    std::shared_ptr<Command> Clone () const {
        return std::make_shared<LogicGate>(*this);
    }


private:
};
typedef std::shared_ptr<LogicGate> ALogicGate;

#endif //PROGRAM_LOGICGATE_H
